/* $Id: Cell_data.cpp,v 1.3 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <vector>
using namespace std;
#include "Cell_data.h"
#include "cd_list.h"
#include "location.h"

Cell_data::Cell_data()
	: cell(NULL),
	  locn_set(false), cell_set(false), neighbors_set(false)
{}


Cell_data::Cell_data(const location &nlocn,
							ICell *ncell,
							const cd_list_t &nneighbors)
	: locn(nlocn),
	  cell(ncell),
	  neighbors(nneighbors),
	  locn_set(true), cell_set(true), neighbors_set(true)
{ assert(ncell); }


location Cell_data::get_location() const
{ assert(locn_set); return locn; }


const ICell* Cell_data::get_cell() const
{ assert(cell_set); return cell.get(); }

ICell* Cell_data::get_cell()
{ assert(cell_set); return cell.get(); }


const cd_list_t* Cell_data::get_neighbors() const
{ assert(neighbors_set); return &neighbors; }

cd_list_t* Cell_data::get_neighbors()
{ assert(neighbors_set); return &neighbors; }

void Cell_data::set_location(const location &nlocn)
{ locn_set = true; locn = nlocn; }


ICell* Cell_data::set_cell(ICell *ncell)
{
	assert(ncell);
	cell_set = true;
	ICell *prev_cell = cell.release(); // NOTE: we do not free prev_cell here
	auto_ptr<ICell> new_cell(ncell);
	cell = new_cell;
	return prev_cell;
}

void Cell_data::set_neighbors(const cd_list_t &nneighbors)
{ neighbors_set = true; neighbors = nneighbors; }


// Private

// DO NOT use this function without first thinking about Cell_data::cell
// copying.
Cell_data::Cell_data(const Cell_data &cd)
{
	assert(false);

	locn      = cd.get_location();

	// This line would would cause to auto_ptrs to point to ths same memory!
	//	cell      = cd.get.get();

	neighbors = cd.neighbors;
}
