/* $Id: Cell_data.h,v 1.4 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __CELL_DATA_H
#define __CELL_DATA_H

#include <vector>
#include <memory>
#include "location.h"
#include "cd_list.h"
#include "ICell.h"


// ICell storage for the simulator,
// contains pointer to the ICell, its location, and its neighbors.
class Cell_data
{
 public:
	Cell_data();
	Cell_data(const location  &nlocn,
				 ICell           *ncell,
				 const cd_list_t &nneighbors);


	location       get_location() const;

	const ICell*   get_cell() const;
	ICell*         get_cell();

	const cd_list_t* get_neighbors() const;
	cd_list_t*       get_neighbors();

	// These functions should only be used by class simulator
 public:
	void set_location(const location &nlocn);
	ICell* set_cell(ICell *ncell); // returns the old cell ptr, callee frees
	void set_neighbors(const cd_list_t &nneighbors);
	
 private:
	location             locn;
	std::auto_ptr<ICell> cell;
	cd_list_t      neighbors;

	bool locn_set, cell_set, neighbors_set;

	Cell_data(const Cell_data &cd);
};

#endif // __CELL_DATA_H
