/* $Id: basic_cell.h,v 1.3 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __BASIC_CELL_H
#define __BASIC_CELL_H

#include "ICell.h"
#include "messenger.h"
#include "shape_sensor.h"
#include "shape_sensor_factory.h"
#include "msg_content.h"
class Cell_data;

class basic_cell : public ICell
{
 public:
	basic_cell(messenger *m, Cell_data *cd, const shape_sensor &ss,
				  const location &locn);

	virtual void step();
	virtual bool done() const;

 private:
	location originating_locn(float signal_strength,
									  float signal_direction) const;

	shape_sensor ssens;
	bool         i_am_done;
	unsigned int steps;
	const location my_locn; // location is only used for debugging
};

class basic_cell_factory
{
 public:
	basic_cell_factory(const shape_sensor_factory &ssf,
							 location::coord_t max_neighbor_dist);

	ICell* operator()(Cell_data *cd, messenger *msger);
							
 private:
	shape_sensor_factory ssf;
};


#include <string>
class basic_msg : public msg_content
{
 public:
	basic_msg();
	basic_msg(const string &s);
	virtual ~basic_msg();
	virtual basic_msg* clone() const;

	std::string text;
};

#endif // __BASIC_CELL_H
