/* $Id: build_cell_layout.cpp,v 1.2 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <stdlib.h>
#include <string>
#include "rand_utils.h"
#include "location.h"
#include "location_neighbor_generator.h"
using namespace std;

int main(int argc, char **argv)
{
	const int  required_num_args = 6;
	const bool verbose_output    = true;

	if( !(required_num_args == argc || 1+required_num_args == argc) )
	{
		cerr << "Usage: build_cell_layout <output_filename>"
			  << " <num_cells> <neighbor_dist> <space_width> <space_height>"
			  << " [prng_seed]" << endl;
		exit(1);
	}

	if(1+required_num_args == argc)
		global_randf().reseed(atoi(argv[required_num_args]));

	const string            out_filename(argv[1]);
	const unsigned int      num_cells(atoi(argv[2]));
	const location::coord_t neighbor_max_dist(atof(argv[3]));
	const location          space_size(atof(argv[4]), atof(argv[5]));

	const location_neighbor_generator lng;

	// Place cells in space
	if(verbose_output)
		cout << "Placing cells in space:" << endl;
	location_neighbor_generator::location_list_t locns;
	assert(lng.generate_locations(num_cells,space_size,locns, verbose_output));

	// Build list of cells' neighbors
	if(verbose_output)
		cout << "Building list of cell's neighbors:" << endl;
	location_neighbor_generator::nghbr_list_list_t nghbrs;
	assert(lng.generate_nghbrs(neighbor_max_dist,locns,nghbrs, verbose_output));

	// Save locns and nghbr relations
	if(verbose_output)
		cout << "Saving locations and neighbor relations:" << endl;
	assert(lng.save(out_filename,space_size, neighbor_max_dist, locns, nghbrs,
						 verbose_output));

	return 0;
}
