/* $Id: location_neighbor_generator.h,v 1.2 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __LOCATION_NEIGHBOR_GENERATOR_H
#define __LOCATION_NEIGHBOR_GENERATOR_H

#include "location.h"
#include <vector>
#include <string>

// Generate locations and, given these locns and the max dist between them,
// which cells are neighbors. Also support saving and loading this data to file
//
// Saving/loading supported because generate_nghbrs()'s runtime O(num_cells^2).
class location_neighbor_generator
{
 public:
	location_neighbor_generator();

	typedef std::vector< location >     location_list_t;
	typedef std::vector< unsigned int > nghbr_list_t;
	typedef std::vector< nghbr_list_t > nghbr_list_list_t;

	bool generate_locations(unsigned int     num_cells,
									const location  &space_size,
									location_list_t &locns,
									bool             output_progress) const;

	bool generate_nghbrs(const location::coord_t &neighbor_max_dist,
								const location_list_t   &locns,
								nghbr_list_list_t       &nghbrs,
								bool                     output_progress) const;


	bool load__nbhr_dist__space_size(const std::string &filename,
												location          &space_size,
												location::coord_t &nmd) const;
	bool load(const std::string &filename,
				 location          &space_size,
				 location::coord_t &nmd,
				 location_list_t   &locns,
				 nghbr_list_list_t &nghbrs,
				 bool               output_progress) const;

	bool save(const std::string       &filename,
				 const location          &space_size,
				 const location::coord_t &neighbor_max_dist,
				 const location_list_t   &locns,
				 const nghbr_list_list_t &nghbrs,
				 bool                     output_progress) const;

 private:
	const std::string delim;
};

#endif // __LOCATION_NEIGHBOR_GENERATOR_H
