/* $Id: locn_tester.cpp,v 1.3 2004/05/11 12:42:49 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "rand_utils.h"
#include "location.h"
#include <iostream>
#include "assert.h"
#include "utils.h"
using namespace std;

void print_data(float dist, float angle,
					 const location &send_locn, const location &recv_locn,
					 const rel_location &rel_locn)
{
	//		cout << "a=" << angle << ", dist=" << dist << endl;
		cout << recv_locn << " \t" << send_locn
			  << ": \t" << rel_locn << endl;
}

void print_recalced_send_locn(const location &recalced_send_locn,
										const location &send_locn)
{
	if( f_eq(recalced_send_locn.get_x(), send_locn.get_x())
		 && f_eq(recalced_send_locn.get_y(), send_locn.get_y()))
			cout << "P";
		else
			cout << "F";
		cout << " -> " << recalced_send_locn << endl;
}

int main(int argc, char **argv)
{
	assert(2 == argc);
	const int N = atoi(argv[1]);
	const float max_coord = 100;
	for(int i=0; i<N; ++i)
   {
		location send_locn(global_randf()(max_coord),
								 global_randf()(max_coord));
		location recv_locn(global_randf()(max_coord),
								 global_randf()(max_coord));

		const float dist  = locn_distance(recv_locn, send_locn);
		const float angle = locn_direction(recv_locn, send_locn);

		const rel_location rel_locn = dist_angle2rel_locn(dist, angle);

		print_data(dist, angle, send_locn, recv_locn, rel_locn);
		

		const location recalced_send_locn(recv_locn.get_x() + rel_locn.get_x(),
													 recv_locn.get_y() + rel_locn.get_y());
		print_recalced_send_locn(recalced_send_locn, send_locn);
	}

	return 1;
}
