/* $Id: rand_utils.cpp,v 1.6 2004/05/11 13:05:14 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "rand_utils.h"
#include <sys/timeb.h>
#include <stdlib.h>
#include <assert.h>

rand_functor::rand_functor()
{
	timeb time;
	ftime(&time);
	rnd_seed = time.time + time.millitm;
	srand(rnd_seed);
}

rand_functor::rand_functor(unsigned int seed)
	: rnd_seed(seed)
{ srand(rnd_seed); }


unsigned int rand_functor::get_seed() const
{ return rnd_seed; }

void rand_functor::reseed(unsigned int seed)
{
	rnd_seed = seed;
	srand(rnd_seed);
}
	
int rand_functor::operator()(int N)
{
	assert(N > 0);
	assert(N < RAND_MAX);
	assert(N-1 > 0);

	int rnd = rand();

	if(N <= rnd)
	{
		rnd = static_cast<int>( N * static_cast<double>(rnd)
			                     / (static_cast<double>(RAND_MAX) + 1) );

		// It's possible that rnd is 0 or N, take care of this:
		// FIXME: this "fix" breaks uniformity!
		if(rnd == 0) ++rnd;
		if(rnd == N) --rnd;
	}
	else if(rnd <= 0)
		rnd = operator()(N);

	assert(0 < rnd && rnd < N);
	return rnd;
}

// WARNING: I've not studied how good of a distribution this creates
double rand_functor::operator()(double N)
{
	return (N * rand()) / static_cast<double>(RAND_MAX);;
}


rand_functor randf;
rand_functor& global_randf()
{ return randf; }
