/* $Id: shape_sensor_factory.cpp,v 1.2 2004/05/09 01:36:20 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "shape_sensor_factory.h"
#include "shape_sensor.h"
#include "location.h"
#include <math.h>
#include <vector>
#include <string>
using namespace std;

shape_sensor_factory::shape_sensor_factory
(const location &space_size,
 const string   &shape_description_filename)
	: map_width(static_cast<unsigned int>(ceil(space_size.get_x()))),
	  map_height(static_cast<unsigned int>(ceil(space_size.get_y())))
{ load_shape_description(shape_description_filename); }


shape_sensor shape_sensor_factory::operator()(const location &sensor_locn)
{
	const unsigned int x = static_cast<unsigned int>(sensor_locn.get_x());
	const unsigned int y = static_cast<unsigned int>(sensor_locn.get_y());
	assert(x <= map_width && y <= map_height);

	shape_sensor ss(shape_map[y][x]);
	return ss;
}


#include "pbm_toolkit.h"

void shape_sensor_factory::load_shape_description
(const string &shape_description_filename)
{
	pbm p(shape_description_filename);
	if(p.get_magic_num() != "P1")
	{
		cerr
			<< "ERROR: Given shape description file is not a Plain PBM, exiting."
			<< endl;
		exit(1);
	}

	shape_map = p.get_image(map_width, map_height);
}
