/* $Id: simulator.h,v 1.3 2004/05/09 01:36:20 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __SIMULATOR_H
#define __SIMULATOR_H

#include "cd_list.h"
#include "scheduler.h"
#include "location.h"
#include "messenger.h"
#include "todeliver_msg.h"
#include <memory>


// CellFactory is used to create new cells, it must have {the | a
// method of the} signature:
// ICell* operator()(Cell_data *cd, messenger *msger);

template<class CellFactory>
class simulator : public messenger
{
 public:
	simulator(const std::string      &locn_nghbr_filename,
				 const location         &space_size,
				 const location::coord_t neighbor_max_dist,
				 CellFactory             cell_fact);
	virtual ~simulator();

	void run();

	const cd_list_t* get_cells() const;
	cd_list_t*       get_cells();

	// messenger interface
 public:
	virtual void deliver_msg(const todeliver_msg &msg,
									 Cell_data           *cd);

 private:
	CellFactory              cell_fact;
	cd_list_t                cells;
	std::auto_ptr<scheduler> cell_scheduler;
	const location::coord_t  neighbor_max_dist;
};

#include "simulator_impl.h"

#endif // __SIMULATOR_H
