/* $Id: utils.cpp,v 1.3 2004/05/09 01:36:20 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "utils.h"
#include <math.h>
#include <sstream>
#include <string>
using namespace std;

#define CONVERT() \
	stringstream ss; \
	ss << x; \
	return ss.str();

string itos(int x)
{ CONVERT() }
string uitos(unsigned int x)
{ CONVERT() }
string ftos(float x)
{ CONVERT() }
string dtos(double x)
{ CONVERT() }

string ptos(int *x)
{
	const unsigned int xui = reinterpret_cast<unsigned int>(x);
	assert(sizeof(x) == sizeof(xui));

	stringstream ss;
	ss << "0x";
	for(int i=2*sizeof(xui) - 1; i>=0; --i)
		ss << "0123456789ABCDEF"[((xui >> i*4) & 0xF)];
	return ss.str();
}



bool f_eq(float x, float y, double tol)
{ return fabs(x-y) <= tol; }

bool f_lt(float x, float y, double tol)
{ return x - y <= tol; }

bool f_le(float x, float y, double tol)
{ return f_eq(x,y) || f_lt(x,y); }

bool f_gt(float x, float y, double tol)
{ return y - x <= tol; }

bool f_ge(float x, float y, double tol)
{ return f_eq(x,y) || f_gt(x,y); }


// C99 has round and rint functions, but my compiler doesn't support c99, so
float round_float(float x)
{
	const float floor_x = floorf(x);
	const float diff = x - floor_x;
 	if(diff >= 0.5)
		return ceilf(x);
 	else
		return floor_x;
}
